###################################################
### chunk number 1: sdm1
###################################################
library(dismo)
#this loads the dismo library
filename <- paste(system.file(package="dismo"), '/ex/bradypus.csv', sep='')
filename
bradypus <- read.table(filename, header=TRUE, sep=',')
head(bradypus)
bradypus <- bradypus[,2:3]
head(bradypus)


###################################################
### chunk number 2: sdm2
###################################################
data(acaule)
dim(acaule)
acgeo <- subset(acaule, !is.na(lat) & !is.na(lon))
dim(acgeo)
acgeo[1:4, c(1:5,7:10)]


###################################################
### chunk number 3: sdm2b
###################################################
library(maptools)
data(wrld_simpl)
plot(wrld_simpl, xlim=c(-130,10), ylim=c(-60,60))
points(acgeo$lon, acgeo$lat, col='red')


###################################################
### chunk number 4: sdm3
###################################################
lonzero = subset(acgeo, lon==0)
lonzero[, 1:13]


###################################################
### chunk number 5: sdm4
###################################################
library(sp)
coordinates(acgeo) = ~lon+lat
ov = overlay(acgeo, wrld_simpl)
cntr = as.character(wrld_simpl@data$NAME[ov])
which(is.na(cntr))
i = which(cntr != acgeo@data$country)
cbind(cntr, acgeo@data$country)[i,]


###################################################
### chunk number 6: sdm5
###################################################
acgeo  = acgeo[coordinates(acgeo)[,'lon'] < 0, ]


###################################################
### chunk number 7: sdm6
###################################################
georef = subset(acaule, (is.na(lon) | is.na(lat)) & ! is.na(locality) )
dim(georef)
georef[1:3,1:13]


###################################################
### chunk number 8: sdm7
###################################################
args(biogeomancer)
b = try(  biogeomancer('Peru', locality=lonzero$locality[3], progress='')  )
b
lonzero$lat[3]


###################################################
### chunk number 9: sdm8
###################################################
files <- list.files(path=paste(system.file(package="dismo"), '/ex', sep=''),
                    pattern='grd', full.names=TRUE )
mask <- raster(files[[1]])
bg <- randomPoints(mask, 500 )
par(mfrow=c(1,2))
plot(!is.na(mask), legend=FALSE)
points(bg, cex=0.5)
# now with an extent
e = extent(-80, -53, -39, -22)
bg2 <- randomPoints(mask, 50, ext=e)
plot(!is.na(mask), legend=FALSE)
plot(e, add=TRUE, col='red')
points(bg2, cex=0.5)


###################################################
### chunk number 10: sdm9
###################################################
files <- list.files(path=paste(system.file(package="dismo"), 
			'/ex', sep=''), pattern='grd', full.names=TRUE )
# The above finds all the files with extension "grd" in the examples
# ("ex") directory of the dismo package. You do not need such a complex
# statement to get your own files
files
predictors <- stack(files)
predictors
layerNames(predictors)
plot(predictors)


###################################################
### chunk number 11: sdm11
###################################################
plot(predictors, 1)
plot(wrld_simpl, add=TRUE)
points(bradypus, col='red', cex=0.5)
points(acgeo, col='blue', pch='x', cex=0.5)


###################################################
### chunk number 12: sdm12
###################################################
presvals <- extract(predictors, bradypus)
backgr <- randomPoints(predictors, 500)
absvals <- extract(predictors, backgr)
pb <- c(rep(1, nrow(presvals)), rep(0, nrow(absvals)))
sdmdata <- data.frame(cbind(pb, rbind(presvals, absvals)))
sdmdata[,'biome'] = as.factor(sdmdata[,'biome'])
head(sdmdata)
tail(sdmdata)
summary(sdmdata)
pairs(sdmdata[,2:5], cex=0.1, fig=TRUE)


###################################################
### chunk number 13: sdm13a
###################################################
m1 = glm(pb ~ bio1 + bio5 + bio12, data=sdmdata)
class(m1)
summary(m1)

m2 = glm(pb ~ ., data=sdmdata)
m2


###################################################
### chunk number 14: sdm13b
###################################################
bc = bioclim(sdmdata[,c('bio1', 'bio5', 'bio12')])
class(bc)
bc
pairs(bc)


###################################################
### chunk number 15: sdm14
###################################################
bio1 = c(40, 150, 200)
bio5 = c(60, 115, 290)
bio12 = c(600, 1600, 1700)
pd = data.frame(cbind(bio1, bio5, bio12))
pd
predict(m1, pd)
predict(bc, pd)


###################################################
### chunk number 16: sdm15
###################################################
p = rnorm(50, mean=0.7, sd=0.3)
a = rnorm(50, mean=0.4, sd=0.4)
par(mfrow=c(1, 2))
plot(sort(p), col='red', pch=21)
points(sort(a), col='blue', pch=24)
legend(1, 0.95 * max(a,p), c('presence', 'absence'), pch=c(21,24), col=c('red', 'blue'))
comb = c(p,a)
group = c(rep('presence', length(p)), rep('absence', length(a)))
boxplot(comb~group, col=c('blue', 'red'))


###################################################
### chunk number 17: sdm16
###################################################
group = c(rep(1, length(p)), rep(0, length(a))) 
cor.test(comb, group)$estimate
mv <- wilcox.test(p,a)
auc <- as.numeric(mv$statistic) / (length(p) * length(a))
auc


###################################################
### chunk number 18: sdm17
###################################################
e = evaluate(p=p, a=a)
class(e)
e
par(mfrow=c(1, 2))
density(e)
boxplot(e, col=c('blue', 'red'))


###################################################
### chunk number 19: sdm19
###################################################
rand <- round(0.75 * runif(nrow(sdmdata)))
traindata <- sdmdata[rand==0,]
traindata <- traindata[traindata[,1] == 1, 2:9]
testdata <- sdmdata[rand==1,]
bc <- bioclim(traindata)
e <- evaluate(testdata[testdata==1,], testdata[testdata==0,], bc)
e
plot(e, 'ROC')


###################################################
### chunk number 20: sdm20
###################################################
pres <- sdmdata[sdmdata[,1] == 1, 2:9]
back <- sdmdata[sdmdata[,1] == 0, 2:9]


###################################################
### chunk number 21: sdm21
###################################################
k <- 5
group <- kfold(pres, k)
group[1:10]
unique(group)


###################################################
### chunk number 22: sdm22
###################################################
e <- list()
for (i in 1:k) {
	train <- pres[group != i,]
	test <- pres[group == i,]
	bc <- bioclim(train)
	e[[i]] <- evaluate(p=test, a=back, bc)
}	


###################################################
### chunk number 23: sdm23
###################################################
auc <- sapply( e, function(x){slot(x, 'auc')} )
auc
mean(auc)
sapply( e, function(x){ x@t[which.max(x@TPR + x@TNR)] } )


###################################################
### chunk number 24: sdm30
###################################################
pred_nf <- dropLayer(predictors, 'biome')


###################################################
### chunk number 25: sdm31
###################################################
group <- kfold(bradypus, 5)
pres_train <- bradypus[group != 1, ]
pres_test <- bradypus[group == 1, ]


###################################################
### chunk number 26: sdm32
###################################################
ext = extent(-90, -32, -33, 23)


###################################################
### chunk number 27: sdm33
###################################################
backg <- randomPoints(pred_nf, n=1000, ext=ext, extf = 1.25)
colnames(backg) = c('lon', 'lat')
group <- kfold(backg, 5)
backg_train <- backg[group != 1, ]
backg_test <- backg[group == 1, ]


###################################################
### chunk number 28: sdm34
###################################################
r = raster(pred_nf, 1)
plot(!is.na(r), col=c('white', 'light grey'), legend=FALSE)
plot(ext, add=TRUE, col='red', lwd=2)
points(backg_train, pch='-', cex=0.5, col='yellow')
points(backg_test, pch='-',  cex=0.5, col='black')
points(pres_train, pch= '+', col='green')
points(pres_test, pch='+', col='blue')


###################################################
### chunk number 29: sdm35
###################################################
bc <- bioclim(pred_nf, pres_train)
plot(bc, a=1, b=2, p=0.85)


###################################################
### chunk number 30: sdm36
###################################################
e <- evaluate(pres_test, backg_test, bc, pred_nf)
e


###################################################
### chunk number 31: sdm37
###################################################
pb <- predict(pred_nf, bc, ext=ext, progress='')
pb
par(mfrow=c(1,2))
plot(pb, main='Bioclim, raw values')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e@t[which.max(e@TPR + e@TNR)]
plot(pb > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')


###################################################
### chunk number 32: sdm38
###################################################
dm <- domain(pred_nf, pres_train)
e <- evaluate(pres_test, backg_test, dm, pred_nf)
e
pd = predict(pred_nf, dm, ext=ext, progress='')
par(mfrow=c(1,2))
plot(pd, main='Domain, raw values')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e@t[which.max(e@TPR + e@TNR)]
plot(pd > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')


###################################################
### chunk number 33: sdm39
###################################################
mm <- mahal(pred_nf, pres_train)
e <- evaluate(pres_test, backg_test, mm, pred_nf)
e
pm = predict(pred_nf, mm, ext=ext, progress='')
par(mfrow=c(1,2))
pm[pm < -10] <- -10
plot(pm, main='Mahalanobis distance')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e@t[which.max(e@TPR + e@TNR)]
plot(pm > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')


###################################################
### chunk number 34: sdm40
###################################################
train <- rbind(pres_train, backg_train)
pb_train <- c(rep(1, nrow(pres_train)), rep(0, nrow(backg_train)))
envtrain <- extract(predictors, train)
envtrain <- data.frame( cbind(pa=pb_train, envtrain) )
envtrain[,'biome'] = factor(envtrain[,'biome'], levels=1:14)
head(envtrain)

testpres <- data.frame( extract(predictors, pres_test) )
testbackg <- data.frame( extract(predictors, backg_test) )
testpres[ ,'biome'] = factor(testpres[ ,'biome'], levels=1:14)
testbackg[ ,'biome'] = factor(testbackg[ ,'biome'], levels=1:14)


###################################################
### chunk number 35: sdm41
###################################################
gm1 <- glm(pa ~ bio1 + bio5 + bio6 + bio7 + bio8 + bio12 + bio16 + bio17, 
			family = binomial(link = "logit"), data=envtrain)
gm2 <- glm(pa ~ bio1 + bio5 + bio6 + bio7 + bio8 + bio12 + bio16 + bio17,
			family = gaussian(link = "identity"), data=envtrain)
e1 = evaluate(testpres, testbackg, gm1)
e2 = evaluate(testpres, testbackg, gm2)
e1
e2


###################################################
### chunk number 36: sdm42
###################################################
pg <- predict(predictors, gm2, ext=ext)
par(mfrow=c(1,2))
plot(pg, main='GLM/gaussian, raw values')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e2@t[which.max(e@TPR + e@TNR)]
plot(pg > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')
points(backg_train, pch='-', cex=0.25)


###################################################
### chunk number 37: sdm46
###################################################
jar <- paste(system.file(package="dismo"), "/java/maxent.jar", sep='')
if (file.exists(jar)) {
	xm <- maxent(predictors, pres_train, factors='biome')
	plot(xm)
} else {
    cat('cannot run this example because maxent is not available on this system')
	plot(1)
}


###################################################
### chunk number 38: sdm47
###################################################
if (file.exists(jar)) {
	e <- evaluate(pres_test, backg_test, xm, predictors)
	e
	px = predict(predictors, xm, ext=ext, progress='')
	par(mfrow=c(1,2))
	plot(px, main='Maxent, raw values')
	plot(wrld_simpl, add=TRUE, border='dark grey')
	threshold <- e@t[which.max(e@TPR + e@TNR)]
	plot(px > threshold, main='presence/absence')
	plot(wrld_simpl, add=TRUE, border='dark grey')
	points(pres_train, pch='+')
} else {	
	plot(1)
}


###################################################
### chunk number 39: sdm51
###################################################
library(randomForest)
model <- pa ~ bio1 + bio5 + bio6 + bio7 + bio8 + bio12 + bio16 + bio17
rf1 <- randomForest(model, data=envtrain)
model <- factor(pa) ~ bio1 + bio5 + bio6 + bio7 + bio8 + bio12 + bio16 + bio17
rf2 <- randomForest(model, data=envtrain)
rf3 <- randomForest(envtrain[,1:8], factor(pb_train))

e = evaluate(testpres, testbackg, rf1)
e
pr <- predict(predictors, rf1, ext=ext)

par(mfrow=c(1,2))
plot(pr, main='Random Forest, regression')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e@t[which.max(e@TPR + e@TNR)]
plot(pr > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')
points(backg_train, pch='-', cex=0.25)


###################################################
### chunk number 40: sdm61
###################################################
library(kernlab)
svm <- ksvm(pa ~ bio1 + bio5 + bio6 + bio7 + bio8 + bio12 + bio16 + bio17, data=envtrain)
e = evaluate(testpres, testbackg, svm)
e
ps <- predict(predictors, rf1, ext=ext)

par(mfrow=c(1,2))
plot(ps, main='Support Vector Machine')
plot(wrld_simpl, add=TRUE, border='dark grey')
threshold <- e@t[which.max(e@TPR + e@TNR)]
plot(ps > threshold, main='presence/absence')
plot(wrld_simpl, add=TRUE, border='dark grey')
points(pres_train, pch='+')
points(backg_train, pch='-', cex=0.25)


